--------------------------------------------------------------------------------
-- Company:        CERN, BE-CO-HT CHANGED
-- Engineer:       Carlos Gil Soriano
-- 
-- Create Date:    18:15:56 11/09/2011 
-- Design Name:    A I2C slave with wishbone slave output and interrupt for MCU
-- Module Name:    i2c_slave_top - Behavioral 
-- Project Name:   CTDAH 
-- Target Devices: 
-- Tool versions: 
-- Description: 
--
-- Dependencies: 
--
-- Revision: 
-- Revision 0.01 - File Created
-- Additional Comments: 
--
----------------------------------------------------------------------------------
library IEEE;
library work;

use IEEE.STD_LOGIC_1164.ALL;
use IEEE.NUMERIC_STD.ALL;
use work.i2c_slave_pkg.ALL;
use work.ctdah_pkg.ALL;

entity i2c_slave_top is
   port (
      sda_oen          : out STD_LOGIC;
      sda_i            : in  STD_LOGIC;
      sda_o            : out STD_LOGIC;
      scl_oen          : out STD_LOGIC;
      scl_i            : in  STD_LOGIC;
      scl_o            : out STD_LOGIC;

      wb_clk           : in  STD_LOGIC;
      wb_rst_i         : in  STD_LOGIC;

      wb_master_stb_o  : out STD_LOGIC;
      wb_master_cyc_o  : out STD_LOGIC;
      wb_master_sel_o  : out STD_LOGIC_VECTOR(3  downto 0);
      wb_master_we_o   : out STD_LOGIC;
      wb_master_data_i : in  STD_LOGIC_VECTOR(31 downto 0);
      wb_master_data_o : out STD_LOGIC_VECTOR(31 downto 0);
      wb_master_addr_o : out STD_LOGIC_VECTOR(15  downto 0);
      wb_master_ack_i  : in  STD_LOGIC;
      wb_master_rty_i  : in  STD_LOGIC;
      wb_master_err_i  : in  STD_LOGIC;

      wb_slave_stb_i   : in  STD_LOGIC;
      wb_slave_cyc_i   : in  STD_LOGIC;
      wb_slave_sel_i   : in  STD_LOGIC_VECTOR(3  downto 0);
      wb_slave_we_i    : in  STD_LOGIC;
      wb_slave_data_i  : in  STD_LOGIC_VECTOR(31 downto 0);
      wb_slave_data_o  : out STD_LOGIC_VECTOR(31 downto 0);
      wb_slave_addr_i  : in  STD_LOGIC_VECTOR(3  downto 0);
      wb_slave_ack_o   : out STD_LOGIC;
      wb_slave_rty_o   : out STD_LOGIC;
      wb_slave_err_o   : out STD_LOGIC;

      pf_wb_addr_o     : out STD_LOGIC;
      rd_done_o        : out STD_LOGIC;
      wr_done_o        : out STD_LOGIC;
      i2c_addr_i       : in  STD_LOGIC_VECTOR(6  downto 0));
end i2c_slave_top;

architecture Behavioral of i2c_slave_top is

   signal s_CTR0_slv   : STD_LOGIC_VECTOR(r_CTR0'a_length - 1 downto 0);
   signal s_CTR0       : r_CTR0;

   signal s_DRXA       : STD_LOGIC_VECTOR(r_DRX'a_length - 1 downto 0);
   signal s_DRXB       : STD_LOGIC_VECTOR(r_DRX'a_length - 1 downto 0);
   signal s_DTX        : STD_LOGIC_VECTOR(r_DTX'a_length - 1 downto 0);

   signal s_load_TX    : STD_LOGIC;
   signal s_pf_wb_addr : STD_LOGIC;
   signal s_rd_done    : STD_LOGIC;
   signal s_wr_done    : STD_LOGIC;

   signal s_clk_i2c    : STD_LOGIC;
   signal s_rst_i2c    : STD_LOGIC;
   signal s_reset_extensor  : STD_LOGIC_VECTOR(2**c_CLK_DIV'length - 1
                                            downto 0) := (others => '1');
begin

   pf_wb_addr_o   <=   s_pf_wb_addr;
   rd_done_o      <=   s_rd_done;
   wr_done_o      <=   s_wr_done;
   s_CTR0         <= f_CTR0(s_CTR0_slv);

   inst_i2c_slave_core: i2c_slave_core
   port map(clk          => wb_clk,  --s_clk_i2c,
            rst_i        => wb_rst_i,--s_rst_i2c,

            sda_oen      => sda_oen,
            sda_i        => sda_i,
            sda_o        => sda_o,
            scl_oen      => scl_oen,
            scl_i        => scl_i,
            scl_o        => scl_o,

            CTR0_i       => s_CTR0_slv,
            DRXA_o       => s_DRXA,
            DRXB_o       => s_DRXB,
            DTX_i        => s_DTX,

            load_tx      => s_load_tx,
            pf_wb_addr_o => s_pf_wb_addr,
            rd_done_o    => s_rd_done,
            wr_done_o    => s_wr_done);

   inst_i2c_regs: i2c_regs
   port map(pf_wb_addr_i => s_pf_wb_addr,
            rd_done_i    => s_rd_done,
            wr_done_i    => s_wr_done, 

            wb_rst_i     => wb_rst_i,
            wb_clk       => wb_clk,

            wb_master_we_o   => wb_master_we_o, 
            wb_master_stb_o  => wb_master_stb_o, 
            wb_master_cyc_o  => wb_master_cyc_o, 
            wb_master_sel_o  => wb_master_sel_o, 
            wb_master_data_i => wb_master_data_i, 
            wb_master_data_o => wb_master_data_o, 
            wb_master_addr_o => wb_master_addr_o, 
            wb_master_ack_i  => wb_master_ack_i, 
            wb_master_rty_i  => wb_master_rty_i, 
            wb_master_err_i  => wb_master_err_i, 

            wb_slave_we_i    => wb_slave_we_i,
            wb_slave_stb_i   => wb_slave_stb_i,
            wb_slave_cyc_i   => wb_slave_cyc_i,
            wb_slave_sel_i   => wb_slave_sel_i,
            wb_slave_data_i  => wb_slave_data_i,
            wb_slave_data_o  => wb_slave_data_o,
            wb_slave_addr_i  => wb_slave_addr_i,
            wb_slave_ack_o   => wb_slave_ack_o,
            wb_slave_rty_o   => wb_slave_rty_o,
            wb_slave_err_o   => wb_slave_err_o,

            CTR0_o           => s_CTR0_slv,  
            DRXA_i           => s_DRXA,
            DRXB_i           => s_DRXB,
            DTX_o            => s_DTX,
            load_tx          => s_load_tx,
            i2c_addr_i       => i2c_addr_i);


   inst_clk_divider : gc_clk_divider
   generic map (g_clk_division_logSize => 4)
   port map (clk_i      => wb_clk,
             rst_i      => wb_rst_i,
             oe_n_i     => '0',
             clk_o      => s_clk_i2c,
             --Divides by twice the value specified
             divider_i  => STD_LOGIC_VECTOR(s_CTR0.CLK_DIV));

   s_rst_i2c <= s_reset_extensor(2**c_CLK_DIV'length - 1);

   --! A shift with reset, consumes just a few SLICEX in Spartan6.
   p_rst_extender : process(wb_clk)
   begin
      if rising_edge(wb_clk) then
         if wb_rst_i = '1' then
            s_reset_extensor <= (others => '1');
         else
            s_reset_extensor(0) <= '0';
            for i in 1 to 2**c_CLK_DIV'length -1 loop
               s_reset_extensor(i) <= s_reset_extensor(i-1);
            end loop;
         end if;
      end if;
   end process;

end Behavioral;
